/* matchbox-nest - a graphical wrapper around xnest
 *
 *  Copyright 2004 Matthew Allum
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <libmb/mb.h>
#include <expat.h>

#include <X11/extensions/XTest.h>

#include "config.h"

typedef struct FakeApp FakeApp;
typedef struct FakeButton FakeButton;

struct FakeApp
{
  Display       *dpy;
  int            screen;
  Window         winmain, winnest, winroot;
  GC             gc;
  MBPixbuf      *pb;

  int            device_width;
  int            device_height;

  int            device_display_x;
  int            device_display_y;
  int            device_display_width;
  int            device_display_height;

  MBPixbufImage *device_img;

  /* Button stuff */

  FakeButton     *button_head;

  struct timeval  key_rep_init_timeout;
  struct timeval  key_rep_timeout;

  /* Xnest  */

  Display        *xnest_dpy;
  char           *xnest_dpy_name;
  char           *xnest_bin_path;

  char           *win_title;
};

struct FakeButton
{
  Window         win;
  
  int            x,y;
  int            width,height;

  KeySym         keysym;

  Pixmap         backing, backing_active;
  MBPixbufImage *img;

  int            repeat;

  FakeButton  *next;
};


int
config_init(FakeApp *app, char *conf_file);

/* buttons */

FakeButton*
button_new(FakeApp *app, int x, int y, int width, int height, KeySym ks, int reepreat, MBPixbufImage *img);

FakeButton*
button_find_from_win(FakeApp *app, Window xevent_window_id);

void
button_press(FakeApp *app, FakeButton *button);

void
button_release(FakeApp *app, FakeButton *button);

void
button_activate(FakeApp *app, FakeButton *button);


/* keys */

#define KEYDOWN   1
#define KEYUP     2
#define KEYUPDOWN 3

int
keys_init(FakeApp *app);

void 
keys_send_key(FakeApp *app, KeySym ks, int key_direction);


